<?php
// $Id: openlayers_ui.layers.inc,v 1.1.2.2 2010/02/05 22:25:44 tmcw Exp $

/**
 * @file
 * This file holds the functions handling layers in the 
 * Openlayers UI.
 *
 * @ingroup openlayers
 */

/**
 * Layer add/edit form.
 */
function openlayers_ui_layers_types(&$form_state, $layer = NULL, $edit = FALSE) {
  $form = array();
  $form['layer_types'] = openlayers_ui_get_layer_type_options('map');
  return $form;
}


/**
 * Layer add/edit form.
 */
function openlayers_ui_layers_form(&$form_state, $layer = NULL) {
  $form = array();

  $form['layer'] = openlayers_ui_get_layer_form($layer);

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Submit handler for layers.
 */
function openlayers_ui_layers_form_submit(&$form, &$form_state) {
  // Save preset
  $layer_class = $form_state['values']['data']['layer_type'];
  $layer = openlayers_layer_type_load($layer_class);

  $layer->name = $form_state['values']['name'];
  $layer->title = $form_state['values']['title'];
  $layer->description = $form_state['values']['description'];
  $layer->data = $form_state['values']['data'];
  $layer->data['resolutions'] = array_map("floatval", array_keys($layer->data['resolutions']));
  rsort($layer->data['resolutions']); 

  $success = $layer->save();

  // Redirect to edit page
  if ($success) {
    drupal_set_message(t('Layer saved.'));
    $form_state['redirect'] = 'admin/settings/openlayers/layers/' . $layer->name . '/edit';
  }
  else {
    form_set_error('openlayers', t('Error trying to save layer.'));
  }
}

/**
 * Layer-wide-settings form
 */

function openlayers_ui_layers_settings(&$form_state, $layer = NULL) {
  $form = array();
  $layer_types = openlayers_layer_types();

  foreach ($layer_types as $name => $type_def) {
    $type = openlayers_layer_type_load($name);
    if (method_exists($type, 'settings_form')) {
      $form[$name] = array(
        '#type' => 'fieldset',
        '#tree' => TRUE,
        '#title' => $type_def['title']);
      $form[$name]['settings'] = $type->settings_form();
    }
  }
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Submit handler for openlayers_ui_layers_settings
 */

function openlayers_ui_layers_settings_submit(&$form, &$form_state) {
  $layer_types = openlayers_layer_types();
  foreach ($form_state['values'] as $key => $value) {
    if (in_array($key, array_keys($layer_types))) {
      foreach ($value['settings'] as $k => $v) {
        variable_set($k, $v);
      }
    }
  }
}

/**
 * Get an actual layer form
 */
function openlayers_ui_get_layer_form($type, $layer = FALSE) {
  $form = array();

  // @TODO: clarify use of data, and of layer_type in overriding it
  $form['data'] = array(
    '#tree' => TRUE,
    '#type' => 'fieldset',
    '#title' => '',
    '#description' => '',
  );
  $form['name'] = array(
    '#type' => 'textfield',
    '#default_value' => isset($type->name) ? $type->name : '',
    '#title' => t('Layer Name'),
    '#description' => t('This is the unique name of the layer. It must contain only alphanumeric characters and underscores.'),
  );
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Layer Title'),
    '#default_value' => isset($type->title) ? $type->title : '',
    '#description' => t('The friendly name of your layer, which will appear in the administration interface.'),
  );
  $form['description'] = array(
    '#type' => 'textfield',
    '#title' => t('Layer Description'),
    '#default_value' => isset($type->description) ? $type->description : '',
  );

  // Arrays merged in order so that options_form can override defaults
  $form['data'] = $type->options_form() + $form['data'];

  return $form;
}

/**
 * Get layer type options.
 */
function openlayers_ui_get_layer_type_options($type) {
  $form = array();
  $form['layer_type_help'] = array(
    '#type' => 'item',
    '#value' => t('Select a layer type to create a new layer.')
  );
  foreach (openlayers_layer_types() as $key => $plugin) {
    $class = ctools_plugin_get_class($plugin, 'layer_type');
    if ($class) {
      // Build form.
      $form[$key] = array(
        '#type' => 'item',
        '#title' => l($plugin['title'], "admin/settings/openlayers/layers/add/{$key}"),
        '#value' =>  $plugin['description'],
      );
    }
  }
  return $form;
}
